###### Standard sub-routines #######

##############################################################################
# Credit card number routines copyright. The code this script uses has been 
# adapted with much thanks from the work of the author following.            
##############################################################################
# Credit card number and expiration date validation routine.
#
# Copyright 1996 furph, Inc.  All Rights Reserved.
#     
# Input:  credit card number, card type, expiration month and year
# Output: "OK", otherwise a (hopefully) reasonable error message string

# Larry Leszczynski (larryl@furph.com)
# furph, Inc. (http://www.furph.com)
# Version 1.0 October 26, 1996

# This routine is used to validate a credit card number, in that it checks
# to see if the number has the correct number of digits and the correct
# checksum for the corresponding credit card type.  It can only tell if the
# number *might* be valid for a given card type - you still have to check
# with the bank to verify that it is a real number - but at least it lets you
# filter out the obviously bogus numbers.
#
# It also checks that the expiration month and year are reasonable
# (i.e. not in the past).
#
# Please notify me via email if you find any errors, or if you find other
# credit card types to add.
#
# Need custom CGI, NSAPI and/or Perl scripts?  Email me!
# Larry (larryl@furph.com)
##############################################################################
# PGP subroutines adapted with much thanks from the work of the author       # 
# following :                                                                #
#                                                                            # 
# Application Name: Extropia's WebResponder                                  #
# Application Authors: Eric Tachibana (Selena Sol) and Gunther Birznieks     #
# Version: 5.0                                                               #  
# http://www.extropia.com                                                    #
##############################################################################
# Email_check and required_check subroutines adapted with much thanks from   #
# the work of the author following :                                         #
#                                                                            #
# Cliff's Form Mailer Version 1.0                                            # 
# Copyright 1998 Shaven Ferret Productions                                   #
# Created 6/4/98                                                             #
# Available at http://www.shavenferret.com/scripts                           #
##############################################################################
########################################################################
# Set Cookie subroutine adapted with much thanks from the work of the  #
# author following:                                                    #
#                                                                      #
# cookietest.pl v1.1; Aug. 18, 1996 Dale Bewley <dlbewley@iupui.edu>   #
# v1.0 29 June 96, v0.9 14 May 96                                      #
#----------------------------------------------------------------------#
# This script and others found at http://www.bewley.net/perl/          #
# Simple cookie demo.                                                  #  
# For more info see:                                                   #
# http://www.bewley.net/perl/cookie-test.html                          #
########################################################################
# Get Cookie sub-routine taken from cookie-lib.txt
# Perl Routines to Manipulate Web Browser Cookies
# kovacsp@egr.uri.edu
# $Id: cookie-lib.txt,v 0.913 1998/11/20 19:45:36 kovacsp Exp $
#
# Copyright (c) 1998 Peter D. Kovacs  
# Unpublished work.
# Portions of this library are taken, without permission (and much 
# appreciated), from the cgi-lib.pl.  You may get that at 
# http://cgi-lib.stanford.edu/cgi-lib
#
# For more information, see:
# http://www.egr.uri.edu/~kovacsp/cookie-lib/
########################################################################
# The weak encryption sub-routine was inspired by code in E-Chat chat 
# room Perl scripts developed by Michael Bagnewski at 
# http://www.e-scripts.com.  
########################################################################

##########
# Uncomment the following lines if you want to test from the command line, eg.
#    ccverify.pl "1234 5678 9012 3456" Visa 9 97
##########
# print &VerifyCC( @ARGV );
# exit;
##########

# Check credit card number basic validity:
sub VerifyCC {

if ($FORM{'Card_number'}) {

   ###########
   # Expecting as input:
   #   $number: numeric string (although spaces, dashes, tabs are ok)
   #   $type:   eg. "VISA", "AMEX", "American Express" (not case sensitive)
   #   $month:  number in the range 1-12
   #   $year:   two or four digit number, eg. "96", "1996", "01", "2001"
   ##########
   
   my( $number, $type, $month, $year ) = @_;

   $number = "$FORM{'Card_number'}"; 
   $type = "$FORM{'Card_type'}"; 
   $month = "$FORM{'Card_expiry_month'}";
   $year = "$FORM{'Card_expiry_year'}";   

   # Quick check that there are reasonable arguments:

   # Check for missing credit card number:   
   unless ( defined $number && $number =~ /\w/ ) {&cc_number_error;}
   # Check for missing credit card type:
   unless ( defined $type   && $type   =~ /\w/ ) {&cc_type_error;}
   # Check for missing expiration month:  
   unless ( defined $month  && $month  =~ /\w/ ) {&cc_month_error;}
   # Check for missing expiration year:
   unless ( defined $year   && $year   =~ /\w/ ) {&cc_year_error;}

   ##########
   # Define the accepted card types and related info:
   ##########

   %cardlength = (
      MASTERCARD      => '16',
      VISA            => '16|13',
      AMEX            => '15',
      AMERICANEXPRESS => '15',
      DINERSCLUB      => '14',
      CARTEBLANCHE    => '14',
      DISCOVER        => '16',
      NOVUS           => '16',
      ENROUTE         => '15',
   );

   %cardprefix = (
      MASTERCARD      => '51|52|53|54|55',
      VISA            => '4',
      AMEX            => '34|37',
      AMERICANEXPRESS => '34|37',
      DINERSCLUB      => '300|301|302|303|304|36|38',
      CARTEBLANCHE    => '300|301|302|303|304|36|38',
      DISCOVER        => '6011',
      NOVUS           => '6011',
      ENROUTE         => '2014|2149',
   );

   ##########
   # Clean up the input parameters and do some basic error checking:
   ##########

   $number =~ s/[ \t-]//g;	# remove spaces, dashes, tabs
    if ( $number =~ /\D/ ) {&cc_number_error;}


   $type = uc( $type );		# uppercase card type
   $type =~ s/[^A-Z]//g;	# remove non-alpha
    
    # Check that only digits are in the month number:
    if ( $month =~ /\D/ ) {&month_digit_error;}

   # Check that the month number is between 1 and 12:
   if ( $month < 1 || $month > 12 ){&cc_month_error;}
   
   # Check that only 2 or 4 digits are entered for the year:
   $length = length( $year );
   if ( $year =~ /\D/ || ( $length != 2 && $length != 4 ) ) {&cc_year_error;}


   # Make year look like "19xx" or "20xx":
   if ( $length == 2 ) {
      $year += 1900;
      $year += 100 if ( $year < 1950 );
   }

   ##########
   # Check:
   #   if the card type is recognized
   #   that the number of digits is ok for that card type
   #   that the number has the expected prefix for that card type
   ##########
   
   #Check for a recognized credit card type:
   unless ( defined $cardlength{$type} ) {&cc_number_error;}


   $length = length( $number );

   @lengths = split( /\|/, $cardlength{$type} );
   $found = 0;
   foreach ( @lengths ) {
      if ( $length == $_ ) {
         $found++;
         last;
      }
   }
  # If there are an invalid number of digits for that card type:
   unless ( $found ) {&cc_number_error;}

   @prefixes = split( /\|/, $cardprefix{$type} );
   $found = 0;
   foreach ( @prefixes ) {
      if ( $number =~ /^$_/ ) {
         $found++;
         last;
      }
   }
  # If there is an invalid prefix for that card type:
        unless ( $found ) {&cc_number_error;}


   ##########
   # Make sure the expiration date is not in the past:
   ##########

   ($sec,$min,$hr,$mday,$mon,$yr,$wday,$yday,$isdst) = localtime(time);
   $mon += 1;	# want 1-12 instead of 0-11
   $yr += 1900;

   # Warning that the expiration date has already passed :
   unless ( $year > $yr || ( $year == $yr && $month >= $mon ) )
   {&cc_year_error;}


   ##########
   # Use LUHN Formula (MOD 10) to validate number:
   ##########

   unless ( $type eq "ENROUTE" ) {	# no checksum

   # Algorithm wants to fuss with alternate digits starting with second
   # from right - we'll reverse so we can work left to right.  That way
   # we don't have to worry about whether the number has an even or odd
   # number of digits.
   @digits = split( //, reverse( $number ) );

   # Define a mapping for the alternate digits
   # (rather than calculate the sums every time):
   @map = ( 0, 2, 4, 6, 8, 1, 3, 5, 7, 9 );

   # Sum the digits, replacing the mapped values for the alternate ones:
   $val = 0;
   for ( $i = 0; $i < $length; $i++ ) {
      $val += $digits[$i++];
      $val += $map[ $digits[$i] ];
   }

   # Valid number must end in 0:
   
   # Warning of invalid card number for that card type:
   unless ( substr( $val, -1, 1 ) == 0 ) {&cc_number_error;}


 # Otherwise number has basic validity
         }
      }  
   
   }

sub cc_number_error {

 &Header;
 print "<CENTER><H3><FONT COLOR=\"\#FF0000\">CREDIT CARD ERROR.</FONT></H3><br></CENTER>\n"; 
 print "<P ALIGN=LEFT>The credit card <b>number</b> is not correct.<br>\n";
 print "Please check it.<br><br>\n"; 
 print "Remove any non-numerical characters if you've used them.<br><br>\n";
 print "Please use your browser's back button and correct this.<br><br>\n\n";
 print "<i>Thank you.</i><br><br></P>\n";
 &Footer;
 exit 0;

 }

sub cc_type_error {

 &Header;
 print "<CENTER><H3><FONT COLOR=\"\#FF0000\">CREDIT CARD ERROR.</FONT></H3><br></CENTER>\n"; 
 print "<P ALIGN=LEFT>The credit card <b>type</b> is not correct.<br>\n";
 print "Please use your browser's back button and change it.<br>\n\n";
 print "<i>Thank you.</i><br></P>\n";
 &Footer;

 exit 0;

 }

sub month_digit_error {

 &Header;
 print "<CENTER><H3><FONT COLOR=\"\#FF0000\">CREDIT CARD ERROR.</FONT></H3><br></CENTER>\n"; 
 print "<P ALIGN=LEFT>The credit card <FONT COLOR=\"\#FF0000\"><b>expiry date</b></FONT>\n";
 print "is not correct.<br><br>\n";
 print "Check the <i>number of digits</i>in the month, and that only numbers are used.<br>\n";
 print "Please use your browser's back button and correct this.<br><br>\n\n";
 print "<i>Thank you.</i></P>\n";
 &Footer;
 
 exit 0;

 }

sub cc_month_error {

 &Header;
 print "<CENTER><H3><FONT COLOR=\"\#FF0000\">CREDIT CARD ERROR.</FONT></H3><br></CENTER>\n"; 
 print "<P ALIGN=LEFT>The credit card <FONT COLOR=\"\#FF0000\"><b>expiry date</b></FONT>\n";
 print " is not correct.<br><br>\n";
 print "Check that the number of <i>months</i> is between <i>1 and 12</i>.<br>\n";
 print "Please use your browser's back button and correct this.<br><br>\n\n";
 print "<i>Thank you.</i></P>\n";
 &Footer;
 
 exit 0;

 }

sub cc_year_error {

 &Header;
 print "<CENTER><H3><FONT COLOR=\"\#FF0000\">CREDIT CARD ERROR.</FONT></H3><br></CENTER>\n"; 
 print "<P ALIGN=LEFT>The credit card <FONT COLOR=\"\#FF0000\"><b>expiry date</b></FONT>\n";
 print "is not correct.<br><br>\n";
 print "Check that the date is not in the past.<br><br>\n";
 print "Please use your browser's BACK button and correct this.<br><br>\n\n";
 print "<i>Thank you.</i></P>\n";
 &Footer;
 
 exit 0;

 }


################### Check for bad password:

sub hack_check {
  
   if($FORM{'password'}) { 
   unless($FORM{'password'} eq "$the_password") {
   &webmaster_alert;
   exit(0);
      }
    }  
  } # End hack check

############################################################

#
# subroutine: make_pgp_file
#   Usage:
#     &make_pgp_file($output_text, $output_file);
# 
#   Parameters:
#     $output_text = unencrypted text that you want to scramble
#     $output_file = name of a file that you will use to
#                    temporarily create the encryption. It
#                    will be removed after it is created
#                    and its contents are assigned to a buffer.
#
#   Output:
#     $pgp_output = the encrypted text that was stored in
#          the $output_file results of running PGP

############################################################

sub make_pgp_file {
  local($output_text, $output_file) = @_;  
  local($pgp_output);

  $ENV{"PGPPATH"} = $pgp_config_files;

  $pgp_command =  "$pgp_path $pgp_options ";

  $pgp_command .= "$pgp_public_key_user_id ";

  $pgp_command .= ">$output_file";

    local($old_path) = $ENV{"PATH"};

    $ENV{"PATH"} = "";

    open (PGPCOMMAND, "|$pgp_command");
 
    $ENV{"PATH"} = $old_path;

  print PGPCOMMAND $output_text;

  close (PGPCOMMAND);

  open(PGPOUTPUT, $output_file);

  while (<PGPOUTPUT>) {

    $pgp_output .= $_;

  } 

  close (PGPOUTPUT);
  unlink($output_file);
 
  unless ($pgp_output) {
     unless ($header_set) {
  print "Content-type: text/html\n\n";
                         } # End unless
  print "<b><H3>Attention webmaster: <FONT COLOR=\"\#FF0000\">PGP is not working properly!</FONT></H3></b><br>\n";
  print "Perhaps the PGP key you submitted in the script setup is invalid,\n";
  print "or you did not specify the correct location of the pubring.pgp and ranseed.bin files.<br><br>\n";
  print "Check your permissions as well.<br><br>\n";
  print "Here is what you put in the script:<br><br>\n";
  print "<b>pgp_config_files:</b> $pgp_config_files<br>\n"; 
  print "<b>PGP key submitted:</b> $pgp_public_key_user_id<br>\n";
  print "<b>Path to PGP program submitted:</b> $pgp_path<br>\n";
  print "<b>Path to temp directory submitted:</b> $pgp_temp_file_path<br>\n";
  print "<b>Raw error information:</b><br>\n$!\n";
     exit(0);  
   }

  return($pgp_output);

}


########### Adapted from Cliff's Form Script: Check for all required fields :

sub required_check {


if (!@required) {
      @required = split(/,/, $FORM{'required_form_fields'});    
    
                       } # End if

foreach $field(@required) {
        unless ($FORM{$field}) {
# Get rid of underscores
$field =~ s/_//g;

&Header;
print "<center><h3>Error - Missing Form Information! </h3><br></center>\n";
print "It seems that you've forgotten to fill out the ";
print "<b><FONT COLOR=\"#FF0000\">$field</FONT></b> "; 
print "field in the form.\n";
print "Please use your browser's back button and correct this.\n";
print "Thank You.</P>\n";
&Footer;
exit;
       }
    }
 }

############## Adapted from Cliff's Form Script: Check the senders' email :

sub email_check {

if ($FORM{'Email'}) {
        unless ($FORM{'Email'} =~ /.+\@.+\..+/) {
 &Header;
print "<h3><center>Email Address Error!</center></h3>\n";
print "The email address that you've entered: "; 
print "<b><FONT COLOR=\"#FF0000\">$FORM{'Email'}</FONT></b>, ";
print "is not quite correct. ";
print "Please use your browser's back button and correct it. ";
print "Thank you.\n";
 &Footer;
 exit;
        }
     }
 }

################## Cookie sub-routines ######

sub setCookie {
	# end a set-cookie header with the word secure and the cookie will only
	# be sent through secure connections
	# local($name, $value, $expiration, $path, $domain, $secure) = @_;

	print "Set-Cookie: ";
	print (ID, "=", $FORM{'ID'}, "; expires=", $expires,
		"; path=", $path, "; domain=", $domain, "; ", $secure, "\n");
     }


sub get_cookie {
  local($chip, $val);
  foreach (split(/; /, $ENV{'HTTP_COOKIE'})) {
    	# split cookie at each ; (cookie format is name=value; name=value; etc...)
    	# Convert plus to space (in case of encoding (not necessary, but recommended)
    s/\+/ /g;
    	# Split into key and value.  
    ($chip, $val) = split(/=/,$_,2); # splits on the first =.
    	# Convert %XX from hex numbers to alphanumeric
    $chip =~ s/%([A-Fa-f0-9]{2})/pack("c",hex($1))/ge;
    $val =~ s/%([A-Fa-f0-9]{2})/pack("c",hex($1))/ge;
    	# Associate key and value
    $cookie{$chip} .= "\1" if (defined($cookie{$chip})); # \1 is the multiple separator
    $cookie{$chip} .= $val;
  }
}

###################

sub hack_alert {
   
    # Stop the script, if unauthorised use of 'GET', 'PUT' or 'HEAD' method :
   
    &Header;
    print "<center><h3><FONT COLOR=\"#FF0000\">Unauthorised Request Method $ENV{'REQUEST_METHOD'}.</FONT></h3></center><br>\n\n";
    print "<P ALIGN=LEFT><i>Stopping this transaction now.</i><br>\n\n";
    print "Webmaster, configure your form with the action POST.</p>\n\n";
    &Footer;
    exit(0);
   }

################### Print output page header text from file


sub Header {

unless($header_set) {
	print "Content-type: text/html\n\n";
          $header_set = 1;
                } # End unless 

	      open (HEAD, "$path_to_data_files/$FORM{'header_file'}");
                
	@head = <HEAD>;
	close (HEAD);
			foreach $line (@head) {
		          	print "$line";
		           } # End foreach
        
	} # End sub

################## Print output page footer text from file:

sub Footer {
	open (FOOT, "$path_to_data_files/$FORM{'footer_file'}");
           
	@foot = <FOOT>;
	close (FOOT);
			foreach $line (@foot) {
			print "$line";
		}
		
	  }

 

############################# Valid referrer check:

sub referer_check {
 $invalid_referer = "1";
  if($ENV{'HTTP_REFERER'}) {
 foreach $referer(@valid_urls) {
	if($ENV{'HTTP_REFERER'} =~ m|https?://([^/]*)$referer|i){
	$invalid_referer = "0";
       last;	
	 }
        }
       }
 if ($invalid_referer eq "1") { 
&Header;
 print "<H2>Error! Bad referer $ENV{'HTTP_REFERER'}.</H2>\n\n";
&Footer;
 exit(0);
      } # End if
  } # End sub
      

### Alert the webmaster by email
sub webmaster_alert {


        print "Content-type: text/html\n\n";
        print  "<B>That is an incorrect password.</B><br>\n";
        print  "The following information has been emailed to the webmaster:<BR>\n";
        print  "Referer       : $ENV{'HTTP_REFERER'}<BR>\n";
        print  "IP Number     : $ENV{'REMOTE_ADDR'}<BR>\n";
        print  "Browser       : $ENV{'HTTP_USER_AGENT'}<BR>\n";
        print  "Proxy (if any): $ENV{'HTTP_FORWARDED'}<BR>\n";
        print  "Server time   : $time $date\n";
      
         

 open (MAIL,"|$mailprog -oi -t") || die "Error - cannot open $mailprog!:$_";
 
 print MAIL ("From: \"Hack Alert Sub-Routine\" <YourScript\@yoursite.net>\n");
 print MAIL ("To: $webmaster\n");
 print MAIL ("Reply-To: <YourScript\@YourSite.net>\n");
 print MAIL ("Subject: Possible Hack Attempt\n\n"); 
 print MAIL "Someone has attempted to access $ENV{'SCRIPT_NAME'}\n";
 print MAIL "without using the correct password.\n";
 print MAIL "Referer       : $ENV{'HTTP_REFERER'}\n";
 print MAIL "IP Number     : $ENV{'REMOTE_ADDR'}\n";
 print MAIL "Browser       : $ENV{'HTTP_USER_AGENT'}\n";
 print MAIL "Proxy (if any): $ENV{'HTTP_FORWARDED'}\n";
 print MAIL "Server time   : $time $date\n";
 print MAIL "\n";
 print MAIL "=" x 60 . "\n";
 print MAIL "\n";

close(MAIL);
exit(0);

   }


# Get the date:

sub get_date {


        ($sec,$min,$hour,$mday,$mon,$year,$wday) = (localtime(time))[0,1,2,3,4,5,6];
        $mon++;
        if ($sec < 10) { $sec = "0$sec"; }
        if ($min < 10) { $min = "0$min"; }
        if ($hour < 10) { $hour = "0$hour"; }
        if ($mday < 10) { $mday = "0$mday"; }
        if ($year < 70){$year = ($year + 2000);}
            else {$year = ($year + 1900);}
        $date = "$mon/$mday/$year";
        $time = "$hour:$min:$sec";

# Get the month as text
@months = ('January','February','March','April','May','June','July','August',
		'September','October','November','December');
$month = $months[$mon];

# Get the day of month as text (PROGRAMMER: Put $mday$append when you 
# want the day of the month as text, in your script, when required)

  if (($mday >= 4 && $mday <= 20) || ($mday >= 24 && $mday <= 30)){$append = "th";}
    elsif ($mday =~ /01|21|31/){$append = "st";}
      elsif ($mday =~ /02|22/){$append = "nd";}
        elsif ($mday =~ /03|23/){$append = "rd";}

 # Day name as text
    @days   = ('Sunday','Monday','Tuesday','Wednesday',
               'Thursday','Friday','Saturday');
    $day = $days[$wday];


         } # End sub


   sub debug {
         
    print "Content-type: text/html\n\n";
    print "Debug sub-routine: Ok up to this point. The code that generated this message is a line like this:<BR><BR>\n\n";
    print "<B>&debug;</B><BR><BR>\n\n";
    print "Get rid of it for the script to proceed normally.<BR>\n\n";
    print "Some output may follow this line:<BR><BR>\n";
    print "$_<BR>\n";
    print "@_<BR>\n";
    print "$!<BR>\n";
    print "$line\n"; 
     
    }



1;